##
##   HBS cosmicray rejection
##                             hbs_crrej4px.cl
##
##        2004.10.13    v 1.0  H.Akitaya
## 
##   usage:
##        hbs_crrej4px (imput file list) (cosmicray threshold) \
##                               (flux ratio) (npass) (window)
##
##   (eg.)
##    cl> hbs_crrej4px px.lst 50.0 10.0 9 7
##
##
##   task definition in login.cl : 
##     task hbs_crrej4px = home$script/hbs_crrej4px.cl
##
##        original and final images : *.px.imh
##                    backup images : *.pxbkup,imh
##            differential images 1 : *.pxdiff1.imh
##            differential images 2 : *.pxdiff2.imh
##            cosmicray mask iamges : *.px.pl
##
##
## note on 
## http://coalsack.astr.tohoku.ac.jp/~akitaya/pukiwiki/
##       pukiwiki.php?HBS%B1%A7%C3%E8%C0%FE%B0%FA%A4%AD%A4%CE%BB%EE%A4%DF2
##

procedure hbs_crrej4px( input_lst, crthresh, crfr, crnp, crwin )

file input_lst { prompt = "Input image list" }
real crthresh { prompt = "task cosmicray param : threshold" }
real crfr { prompt = "task cosmicray param : flux ratio" }
int crnp { prompt = "task cosmicray param : npass"}
int crwin { prompt = "task cosmicray param : win"}

begin

string bkup_lst, diff1_lst, diff2_lst, pl_lst, px_lst, log_fn, tmp_txt

####  cosmicray parameters #####
#real crthresh, crfr
#int crnp, crwin
# recommended parameters for OAO74 TTSs (2004.10.13)
# crthresh = 50.0 or 40.0
# crfr = 10.0
# crnp = 9
# crwin = 7
#

##### UNIX command 'sed' will be used in this script ####
task sed = "$foreign"

################################
# file list definition
################################

# original (*.px.imh) file list
px_lst = input_lst

# backup file list
bkup_lst = "hbs_crrej4px_bkup.lst"

# cosmicray mask file list
pl_lst = "hbs_crrej4px_pl.lst"

# differential image 1 (original - cosmicrayed ) list
diff1_lst = "hbs_crrej4px_diff1.lst"

# differential image 2 (original - cosmicrayed - cr mask pixfixed) list
diff2_lst = "hbs_crrej4px_diff2.lst"

# log file name
log_fn = "hbs_crrej4px.log"
tmp_txt = "hbs_crrej4px_temp.txt"

# write time stump into logfile
time( >> log_fn )
printf( "Log File : %s\n", log_fn )

####################################
#       pre-process
####################################
#
# ####### delete old files ########
#
if( access( bkup_lst ) )
{
  imdelete( "@"//bkup_lst, verify- )
  delete( bkup_lst )
}
if( access( pl_lst ) )
{
  imdelete( "@"//pl_lst, verify- )
  delete( pl_lst )
}
if( access( diff1_lst ) )
{
  imdelete( "@"//diff1_lst, verify- )
  delete( diff1_lst )
}
if( access( diff2_lst ) )
{
  imdelete( "@"//diff2_lst, verify- )
  delete( diff2_lst )
}

#
# ####### make list files ########
#

sed ( 's/px/pxbkup/', px_lst, > bkup_lst )
sed ( 's/px/pxdiff1/',  px_lst, > diff1_lst )
sed ( 's/px/pxdiff2/',  px_lst, > diff2_lst )
sed ( 's/imh/pl/', px_lst, > pl_lst )

#
# ####### backup original images ########
#
 
 imrename ( "@"//px_lst, "@"//bkup_lst )

###################################################
#       main 1: rejection by task "cosmicray"
###################################################
#
# ####### cosmicray rejection and make cr-mask images ########
#

 cosmicray ( "@"//bkup_lst, "@"//px_lst, \
    crmasks="@"//pl_lst, \
    threshold = crthresh, \
    fluxratio=crfr, npasses=crnp, window=crwin, inter- )

# ----- write parameters into logfile and stdout ------
 print( "# cosmicray - threshold  : ", crthresh, >> log_fn)
 print( "# cosmicray - flux ratio : ", crfr, >> log_fn)
 print( "# cosmicray - npasses    : ", crnp, >> log_fn)
 print( "# cosmicray - window     : ", crwin, >> log_fn)
#
 print( "# cosmicray - threshold  : ", crthresh)
 print( "# cosmicray - flux ratio : ", crfr)
 print( "# cosmicray - npasses    : ", crnp)
 print( "# cosmicray - window     : ", crwin)

# ----- write infomation about rejected pixels ------
# check
 imarith ( "@"//bkup_lst, "-", "@"//px_lst, "@"//diff1_lst )
 imstat ( "@"//diff1_lst, lower=0.00001,\
  >> log_fn )

# if you need ...
#hbsdispl crrej_diff1.lst

###################################################
#       main 2: rejection by task "fixpix"
###################################################
#
# ####### cosmicray rejection by fixpix based on cr-mask images ########
#

 fixpix ( "@"//px_lst, "@"//pl_lst, \
  linterp="1",  cinterp="2", verbose+, pixels+,
    >> log_fn )
   
# ----- write infomation about rejected pixels ------
 imarith ( "@"//bkup_lst, "-" , "@"//px_lst, "@"//diff2_lst )
 imstat ( "@"//diff2_lst, lower=0.00001,\
  >> log_fn )

# if you need ...
# hbsdispl crrej_diff2.lst

# delete
#imdelete ( "@"\\bkup_lst )
#imdelete ( "@"\\pl_lst )
#imdelete ( "@"\\diff1_lst )
#imdelete ( "@"\\diff2_lst )

end
