#
#       HBS REDUCTION SUB ROUTINE
#
#       rfits all '*.FIT' in the calibration directory : hbscalrfits.cl
#
#      *.FIT --> *.imh,*.pix
#
#        1998.06.13.     K. Kawabata
#        2005/04/07      M.Isogai     for iraf v2.12
#        2008/11/19      M.Isogai     for iraf v2.14
#

procedure hbsrfits

begin

  file fn
  string inlist, outlist
  int n_version      # IRAF version (210 or 211)
  int c
  string buf1, buf2

#--- initial setting

  inlist = "hbs.tmp.in"
  outlist = "hbs.tmp.out"

  if( access( inlist ) == 1 )
    delete( inlist, ver- )
  if( access( outlist ) == 1 )
    delete( outlist, ver- )


#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  else
    {
      n_version = 211
    }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n"
    printf( "cl> set version=V2.10  (V2.10 and V2.11 are valid)\n"
    bye
  }

#--- check .FIT files

  chdir _caldir$

  if( access( inlist ) == 1 )
    delete( inlist, ver- )
  sections( "*.FIT", option="fullname", >& "dev$null" )

  back

  if( sections.nimage > 0 )
  {

#--- rfits .FIT files

    chdir _caldir$

    sections( "*.FIT", option="fullname", > inlist )
    sed( 's/\.FIT//', inlist, > outlist )

    list = outlist

    while( fscan( list, fn ) != EOF )
    {
      if( access( fn//".imh" ) == 1 )
        next

      printf( "%s  ", fn )
      if( n_version == 210 )
      { 
        rfits( fn//".FIT", "", fn, make_image+, long_header-, short_header-,\
               datatype="real", blank=0., scale-, oldirafname-, offset=0 )
      }
      else
      {
        if( n_version == 211 )
        {
          rfits( fn//".FIT", "", fn//".imh", make_image+, long_header-,\
  	         short_header-,\
                 datatype="real", blank=0., scale-, oldirafname-, offset=0 )
        }
        else
        {
          printf( "IRAF version erorr (%d)\n", n_version )
          bye
        }
      }
      delete( fn//".FIT", ver- )
    }
    printf( "\n" )
    delete( inlist, ver- )
    delete( outlist, ver- )

    back
  }
 
end
