#
#       HBS REDUCTION SUB ROUTINE
#
#       Spectrum correction for cross-talk and 2nd-order light: hbscntcor.cl
#
#       2000/02/29      K.Kawabata  Ver.3.00
#       2005/04/07      M.Isogai    Ver.3.10 for iraf v2.12
#       2008/11/19      M.Isogai    Ver.3.20 for iraf v2.14
#

procedure hbscntcor

begin

  file listoflist
  string hbslist[500]
  string shorttyp[500], title[500]
  string exptime[500], diaph[500], filter[500]
  string slit[500]
  int c, i, j, n, n_list
  int flag_uplow  #  0:lower  1: upper  -1:other

  int flag_2o = 1
  int flag_ct = 1

  file crostk1, crostk2, crostk3
  file secord
  int nccd, ntel
  string calmode, somode, ctmode, calstr

  file fn
  string fn_stack[500]
  int n_image
  string fnobe1, fnobo1, fnobe2, fnobo2
  string fnobe, fnobo
  string fnnexte, fnnexto
  file tmp_fn1, tmp_fn2, tmp_list
  file fnctr1, fnctr2 # coefficients with cross-talk ratios
  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7
  string prmfile
  file logfn

  file caldir, fn_def, etcdir

  int flag_erase     # 0:keep old images, 1:erase old images
  int flag_inquiry   # 1:'flag_overwrite' applys to all image
  int flag_overwrite # 0:skip 1:overwrite
  int flag_ver       # 0:no display 1:display
  int n_version      # IRAF version (210 or 211)

#--- initial settings

  logfn = envget( "_logfile" )

  if( access( "hbs.tmp.time" ) == 1 )
    delete( "hbs.tmp.time", ver- )
  time >> hbs.tmp.time
  list = "hbs.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "hbs.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "---------------------------" )
  print( " Spectral count correction" )
  print( "---------------------------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "---------------------------", >> logfn )
  print( " Spectral count correction", >> logfn )
  print( "---------------------------", >> logfn )
  print( "", >> logfn )

  listoflist = envget( "_rootlist" )
  caldir = envget( "_caldir" )
  etcdir = envget( "_etcdir" )
  fn_def = caldir//envget( "_defhname" )
  crostk1 = caldir//envget( "_crostk" )//".1"
  crostk2 = caldir//envget( "_crostk" )//".2"
  crostk3 = caldir//envget( "_crostk" )//".3"
  secord = caldir//envget( "_2ndord" )

  fnctr1 = "hbs.tmp.ctr1"
  fnctr2 = "hbs.tmp.ctr2"

  tmp_fn1 = mktemp( "hbs.tmp.image." )
  tmp_fn2 = mktemp( "hbs.tmp.image." )
  tmp_list = mktemp( "hbs.tmp.list." )

  if( access( secord ) == 0 )
  {
    printf( "Second-order spectrum data '%s' not found.\n", secord )
    printf( "Skip second-order correction.\n" )
    printf( "Second-order spectrum data '%s' not found.\n", secord,\
    >> logfn )
    printf( "Skip second-order correction.\n", >> logfn )
    flag_2o = 0
  }

    if( access( crostk1//".imh" ) == 0 )
    {
      printf( "Cross-talk ratio spectrum '%s' not found.\n", crostk1 )
      printf( "Skip cros-talk correction.\n" )
      printf( "Cross-talk ratio spectrum '%s' not found.\n", crostk1,\
      >> logfn )
      printf( "Skip cros-talk correction.\n", >> logfn )
      flag_ct = 0
    }
    if( access( crostk2//".imh" ) == 0 )
    {
      printf( "Cross-talk ratio spectrum '%s' not found.\n", crostk2 )
      printf( "Skip cros-talk correction.\n" )
      printf( "Cross-talk ratio spectrum '%s' not found.\n", crostk2,\
      >> logfn )
      printf( "Skip cros-talk correction.\n", >> logfn )
      flag_ct = 0
    }
    if( access( crostk3//".imh" ) == 0 )
    {
      printf( "Cross-talk ratio spectrum '%s' not found.\n", crostk3 )
      printf( "Skip cros-talk correction.\n" )
      printf( "Cross-talk ratio spectrum '%s' not found.\n", crostk3,\
      >> logfn )
      printf( "Skip cros-talk correction.\n", >> logfn )
      flag_ct = 0
    }


#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    printf( "Cannot find '%s'\n", fn_def, >> logfn )
    bye
  }

  nccd = 1        # for old 'hbsdefine.h'
  ntel = 1        # for old 'hbsdefine.h'  1: DAO36, 2: other

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "NCCD" )
      nccd = int( buf3 )
    if( buf2 == "OBSERVATORY" )
    {
print( buf3 )
      if( substr( buf3, 1, 7 ) == "OKAYAMA" )
        ntel = 2
    }
  }

  if( nccd == 1 && ntel == 1 )
  {
    printf( "These images are taken by T5 cryocam at Dodaira. Correction skipped.\n" )
    printf( "These images are taken by T5 cryocam at Dodaira. Correction skipped.\n",\
    >> logfn )
    flag_ct = 0
    flag_2o = 0
  }

#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n", >> logfn )
      bye
    }
  }

#----- Replace Mode

  if( envget( "_fg_mode" ) == "1" )
  {
    flag_inquiry = 1
    flag_overwrite = 1
  }
  else
  {
    if( envget( "_fg_mode" ) == "2" )
    {
      flag_inquiry = 1
      flag_overwrite = 0
    }
    else
    {
      if( envget( "_fg_mode" ) == "3" )
      {
        flag_inquiry = 0
        flag_overwrite = 0
      }
      else
      {
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n" )
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n",\
                >> logfn )
        bye
      }
    }
  }
    
#----- Erase/Reserve old image

  if( envget( "_fg_del2" ) == "1" )
  {
    flag_erase = 1
  }
  else
  {
    if( envget( "_fg_del2" ) == "2" )
    {
      flag_erase = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_del2) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_del2) Retry.\n", >> logfn )
      bye
    }
  }

#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  else
    {
      n_version = 211
    }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n" )
    printf( "cl> set version=V2.10  (V2.10 and V2.11 are valid)\n" )
    printf( "IRAF version check failed.\n", >> logfn )
    printf( "Please input like next line, and restart 'hbsred'\n", >> logfn )
    printf( "cl> set version=V2.10  (V2.10 and V2.11 are valid)\n", >> logfn )
    bye
  }

#----- Scanning the list file

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//".   Please do hbsgroup." )
    print( "Cannot find "//listoflist//".   Please do hbsgroup.", >> logfn )
    return
  }

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL",\
             >> logfn )
      break
    }
    if( buf2 != "OBJ" && buf2 != "COM" )
      next
    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- main routine

  for( i=1; i<=n_list; i+=1 )
  {
    if( shorttyp[i] != "DAR" && shorttyp[i] != "BIA" &&\
        shorttyp[i] != "NEA" )
    {
      prmfile = hbslist[i]//".prm"

#----- check diaphragm

      c = strlen( diaph[i] )
      if( substr( diaph[i], 4, 4 ) == "L" )
      {
        flag_uplow = 0
        if( flag_ver == 1 )
          printf( "--- OBJECT LIST : %s  LOWER ---\n",\
	          hbslist[i] )
        printf( "--- OBJECT LIST : %s  LOWER ---\n",\
	        hbslist[i], >> logfn )
      }
      else
      {
        if( substr( diaph[i], 4, 4 ) == "U" )
        {
          flag_uplow = 1
          if( flag_ver == 1 )
            printf( "--- OBJECT LIST : %s  UPPER ---\n",\
	            hbslist[i] )
          printf( "--- OBJECT LIST : %s  UPPER ---\n",\
	          hbslist[i], >> logfn )
        }
        else
        {
          flag_uplow = -1
          printf( "%s did not used two diaphragm mode\n", hbslist[i] )
          printf( "%s did not used two diaphragm mode\n", hbslist[i],\
                  >> logfn )
	  next
        }
      }

      list = prmfile
      if( fscan( list, buf, buf, buf, buf, calmode, buf, buf, buf ) != 8 )
      {
        printf( "Read error in '%s'\n", prmfile )
        printf( "Read error in '%s'\n", prmfile, >> logfn )
        bye
      }
      somode = substr( calmode, 2, 2 )
      ctmode = substr( calmode, 3, 3 )

      if( nccd == 1 && ntel == 1 ) # for TI CCD
      {
        somode = "2"
        ctmode = "2"
      }
      if( flag_2o == 0 )
        somode = "2"
      if( flag_ct == 0 )
        ctmode = "2"

      if( ctmode == "1" )
        calstr = "Mode: Cross-talk cor. ON"
      else
        calstr = "Mode: Cross-talk cor. OFF"

      if( somode == "1" )
        calstr = calstr//", 2nd-ord cor. ON"
      else
        calstr = calstr//", 2nd-ord cor. OFF"

      if( flag_ver == 1 )
      {
        printf( "%s\n", calstr )
      }
      printf( "%s\n", calstr, >> logfn )


#----- calculate correction coefficient from cross-talk ratios

      if( ctmode == "1" )
      {
        if( access( fnctr1//".imh" ) == 1 )
          imdelete( fnctr1, ver- )
        if( access( fnctr2//".imh" ) == 1 )
          imdelete( fnctr2, ver- )

	if( access( tmp_fn1//".imh" ) == 1 )
	  imdelete( tmp_fn1, ver- )

        imarith( 1.0, "-", crostk2, tmp_fn1,\
                 pixtype="real", calctype="real", ver- )
        imarith( 1.0, "/", tmp_fn1, fnctr1,\
                 pixtype="real", calctype="real", ver- )
        imdelete( tmp_fn1, ver- )

        imarith( crostk1, "-", crostk3, tmp_fn1,\
                 pixtype="real", calctype="real", ver- )
        imarith( tmp_fn1, "*", fnctr1, fnctr2,\
                 pixtype="real", calctype="real", ver- )
        imdelete( tmp_fn1, ver- )
      }

#---- main routine

      sed( 's/fit\.imh//', hbslist[i], > tmp_list )

      n_image = 0
      list = tmp_list

      while( fscan( list, buf ) != EOF )
      {
        n_image += 1
        fn_stack[n_image] = buf
      }
      delete( tmp_list, ver- )

      for( j=1; j<=n_image; j+=1 )
      {
        fn = fn_stack[j]

        fnobe1 = fn//"od.e"
        fnobo1 = fn//"od.o"
        fnobe2 = fn//"tmp.e"
        fnobo2 = fn//"tmp.o"
        fnobe = fn//"co.e"
        fnobo = fn//"co.o"

        if( j < n_image )
	{
          fnnexte = fn_stack[j+1]//"co.e"
          fnnexto = fn_stack[j+1]//"co.o"
	}
	else
	{
          fnnexte = "none"
          fnnexto = "none"
	}

        if( access( fnobe1//".imh" ) == 0 || access( fnobo1//".imh" ) == 0 )
        {
          printf( "%s--(not exist. skip.)  ", fn )
          printf( "%s--(not exist. skip.)  ", fn, >> logfn )
          next
        }	

        if( access( fnobe//".imh" ) == 1 || access( fnobo//".imh" ) == 1 )
        {
          if( flag_overwrite == 0 )
          {
            if( fnnexte != "none" && ( access( fnnexte//".imh" ) == 0 ||\
	        access( fnnexto//".imh" ) == 0 ) )
            {
	      printf( "\n%s: Maybe insufficient proccess last time.\n", fn )
	      printf( "\n%s: Maybe insufficient proccess last time.\n",\
                      fn, >> logfn )
              goto inquiry
            }

            if( flag_inquiry == 1 )
            {
              if( flag_ver == 1 )
                printf( "%s--(already exists. skip.)  ", fn )
              printf( "%s--(already exists. skip.)  ", fn, >> logfn )
              next
            }
            else
	    {
              printf( "\n%s already exist.\n", fn )
              printf( "\n%s already exist.\n", fn, >> logfn )
          inquiry:
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> " )
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> ", >> logfn )
              c = scan( buf )
              while( buf != "Y" && buf != "y" && buf != "n" && buf != "N" )
                c = scan( buf )
              if( buf == "Y" )
              {
                flag_inquiry = 1
                flag_overwrite = 1
              }
              if( buf == "N" )
              {
                flag_inquiry = 1
                flag_overwrite = 0
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn )
                printf( "%s--(already exists. skip.)  ", fn, >> logfn )
                next
              }
              if( buf == "n" )
              {
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn )
                printf( "%s--(already exists. skip.)  ", fn, >> logfn )
                next
              }
            }
          }
          if( flag_ver == 1 )
            printf( "%s(overwrite)  ", fn )
          printf( "%s(overwrite)  ", fn, >> logfn )
        }
        else
        {
          if( flag_ver == 1 )
            printf( "%s  ", fn )
          printf( "%s  ", fn, >> logfn )
        }

        if( access( fnobe2//".imh" ) == 1 )
	  imdelete( fnobe2, ver- )
        if( access( fnobo2//".imh" ) == 1 )
	  imdelete( fnobo2, ver- )
        if( access( fnobe//".imh" ) == 1 )
	  imdelete( fnobe, ver- )
        if( access( fnobo//".imh" ) == 1 )
	  imdelete( fnobo, ver- )

#--- Skip this process in case of no correction to be appllied

        if( ctmode == "2" && somode == "2" )
	{
	  if( flag_ver == 1 )
	    printf( "No correction applied.\n" )
	  printf( "No correction applied.\n", >> logfn )
	  imcopy( fnobe1, fnobe, ver- )
	  imcopy( fnobo1, fnobo, ver- )
	  next
	}

#--- Correction for cross-talk components

        if( ctmode == "1" )
	{
	  if( flag_ver == 1 )
	    printf( "Cross-talk... " )
	  printf( "Cross-talk... ", >> logfn )
	
          if( flag_uplow == 0 ) # lower hole
	  {
	    if( access( tmp_fn1//".imh" ) == 1 )
	      imdelete( tmp_fn1, ver- )

            imarith( fnobo1, "*", fnctr2, tmp_fn1,\
                     pixtype="real", calctype="real", ver- )
            imarith( fnobe1, "*", fnctr1, tmp_fn2,\
                     pixtype="real", calctype="real", ver- )
            imarith( tmp_fn2, "-", tmp_fn1, fnobe2,\
                     pixtype="real", calctype="real", ver- )
	    imdelete( tmp_fn1, ver- )
	    imdelete( tmp_fn2, ver- )

            imarith( fnobo1, "*", fnctr1, fnobo2,\
                     pixtype="real", calctype="real", ver- )
	  }
	  else # upper hole
	  {
	    if( access( tmp_fn1//".imh" ) == 1 )
	      imdelete( tmp_fn1, ver- )

            imarith( fnobe1, "*", fnctr1, fnobe2,\
                     pixtype="real", calctype="real", ver- )

            imarith( fnobe1, "*", fnctr2, tmp_fn1,\
                     pixtype="real", calctype="real", ver- )
            imarith( fnobo1, "*", fnctr1, tmp_fn2,\
                     pixtype="real", calctype="real", ver- )
            imarith( tmp_fn2, "-", tmp_fn1, fnobo2,\
                     pixtype="real", calctype="real", ver- )
	    imdelete( tmp_fn1, ver- )
	    imdelete( tmp_fn2, ver- )
	  }
	  if( flag_ver == 1 )
	    printf( "Done., " )
	  printf( "Done., ", >> logfn )
	}
	else
	{
	  if( flag_ver == 1 )
	    printf( "Cross-talk skipped., " )
	  printf( "Cross-talk skipped., ", >> logfn )
	  imcopy( fnobe1, fnobe2, ver- )
	  imcopy( fnobo1, fnobo2, ver- )
	}

#--- 2nd-order spectrum correction

        if( somode == "1" )
	{
	  if( flag_ver == 1 )
	    printf( "2nd-ord sp. ... " )
	  printf( "2nd-ord sp. ... ", >> logfn )

          if( access( fnobe2 ) == 1 )
	    delete( fnobe2, ver- )
          if( access( fnobo2 ) == 1 )
	    delete( fnobo2, ver- )
          if( access( fnobe ) == 1 )
	    delete( fnobe, ver- )
          if( access( fnobo ) == 1 )
	    delete( fnobo, ver- )

          if( access( fnobe//".imh" ) == 1 )
	    imdelete( fnobe, ver- )
          if( access( fnobo//".imh" ) == 1 )
	    imdelete( fnobo, ver- )

          if( n_version == 210 )
          { 
            wfits( fnobe2//".imh", fnobe2, bscale=1.0, bzero=0.0, long_header-,
                   short_header-, bitpix=-32, scale-, autoscale- )
            wfits( fnobo2//".imh", fnobo2, bscale=1.0, bzero=0.0, long_header-,
                   short_header-, bitpix=-32, scale-, autoscale- )
          }
          else
	  {
            if( n_version == 211 )
            {
              wfits( fnobe2//".imh", fnobe2, bscale=1.0, bzero=0.0, long_header-,
                     fextn="", short_header-, bitpix=-32, scale-, autoscale- )
              wfits( fnobo2//".imh", fnobo2, bscale=1.0, bzero=0.0, long_header-,
                     fextn="", short_header-, bitpix=-32, scale-, autoscale- )
            }
            else
            {
              printf( "IRAF version erorr (%d)\n", n_version )
              printf( "IRAF version erorr (%d)\n", n_version, >> logfn )
              bye
            }
	  }
	  imdelete( fnobe2, ver- )
	  imdelete( fnobo2, ver- )

          copy( fnobe2, fnobe, ver- )
	  copy( fnobo2, fnobo, ver- )

	  hbs2ossub( fnobe2, secord, fnobe )
	  hbs2ossub( fnobo2, secord, fnobo )

	  delete( fnobe2, ver- )
	  delete( fnobo2, ver- )

          if( n_version == 210 )
          {
            rfits( fnobo, "", fnobo, make_image+, long_header-, short_header-,\
                   datatype="real", blank=0., scale-, oldirafname-, offset=0 )
            rfits( fnobe, "", fnobe, make_image+, long_header-, short_header-,\
                   datatype="real", blank=0., scale-, oldirafname-, offset=0 )
          }
          if( n_version == 211 )
          {
            rfits( fnobo, "", fnobo//".imh", make_image+, long_header-,\
	           short_header-,\
                   datatype="real", blank=0., scale-, oldirafname-, offset=0 )
            rfits( fnobe, "", fnobe//".imh", make_image+, long_header-,\
	           short_header-,\
                   datatype="real", blank=0., scale-, oldirafname-, offset=0 )
          }
	  delete( fnobe, ver- )
	  delete( fnobo, ver- )

	  imreplace( fnobe, 0.0, lower=INDEF, upper=0.0 )
	  imreplace( fnobo, 0.0, lower=INDEF, upper=0.0 )

          hedit( fnobe, "HISTORY", "Spectrum correction ("//calstr//")",
            delete-, verify-, show-, update+ )
          hedit( fnobo, "HISTORY", "Spectrum correction ("//calstr//")",
            delete-, verify-, show-, update+ )

	  if( flag_ver == 1 )
	    printf( "Done." )
	  printf( "Done.", >> logfn )
        }
	else
	{
	  if( flag_ver == 1 )
	    printf( "2nd-ord sp. skipped." )
	  printf( "2nd-ord sp. skipped.", >> logfn )

	  imrename( fnobe2, fnobe, ver- )
	  imrename( fnobo2, fnobo, ver- )

          hedit( fnobe, "HISTORY", "Avoiding spectrum correction",
            delete-, verify-, show-, update+ )
          hedit( fnobo, "HISTORY", "Avoiding spectrum correction",
            delete-, verify-, show-, update+ )
	}

        if( flag_ver == 1 )
          printf( "\n" )
        printf( "\n", >> logfn )
      }
      if( access( fnctr1//".imh" ) == 1 )
        imdelete( fnctr1, ver- )
      if( access( fnctr2//".imh" ) == 1 )
        imdelete( fnctr2, ver- )
    }
  }
  

end
