#
#       HBS REDUCTION SUB ROUTINE
#
#       make table of I, q, u, p, theta
#
#       1997/09/10      K.Kawabata ver.1.00
#       1997/09/11      K.Kawabata ver.1.10  gtcorl(r).xy. spavel(r).xy
#       1998/06/11      K.Kawabata ver.2.10
#       2000/02/29      K.Kawabata ver.3.00 for SITe-CCD
#       2005/04/07      M.Isogai   Ver.3.10 for iraf v2.12
#       2008/11/19      M.Isogai   Ver.3.20 for iraf v2.14
#
#
procedure hbsmktable

begin

  file grouplist

  string fn_head0[300], diaph0[300], binfile0[300], fn_cmnt0[300]

  file binfile
  string fn_head, fnphot
  string diaph

  file fn_i1ave, fn_i1sig
  file fn_i2ave, fn_i2sig
  file fn_qave, fn_qsig
  file fn_uave, fn_usig
  file fn_out1, fn_out2
  file fn_k0ave, fn_k1ave, fn_k2ave, fn_k3ave
  file fn_kn
  file logfn
  int n_list
  int c
  int flag_i, flag_u, flag_q
  string rdir, etcdir, upfile, gtfile, spfile
  string uphead
  string gthead
  string sphead
  int flag_mode = 0  # 0:equatorial 1:instrumental
  string buf, buf1, buf2
  int n_version      # IRAF version (210 or 211)
  file tmp_iqu
  string obslog
  real jd, am, exptime
  int n_i, n_q, n_u

  file fn_cmnt
  string cmnt

#--- initial settings

  logfn = envget( "_logfile" )

  if( access( "hbs.tmp.time" ) == 1 )
    delete( "hbs.tmp.time", ver- )
  time >> hbs.tmp.time
  list = "hbs.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "hbs.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "---------------------------------" )
  print( " make table of I, q, u, P, theta " )
  print( "---------------------------------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "---------------------------------", >> logfn )
  print( " make table of I, q, u, P, theta ", >> logfn )
  print( "---------------------------------", >> logfn )
  print( "", >> logfn )

  rdir = envget( "_caldir" )
  etcdir = envget( "_etcdir" )
  grouplist = envget( "_grouplist" )
  uphead = envget( "_uphead" )
  gthead = envget( "_gthead" )
  sphead = envget( "_sphead" )
  obslog = envget( "_avelog" )
  fnphot = rdir//envget( "_photmet" )
  tmp_iqu = "HBS.iqu.tmp"

  buf = ""
  n_i = 0
  n_q = 0
  n_u = 0


  printf( "Table type   [1]:equatorial  [2]:instrumental >> " )
  printf( "Table type   [1]:equatorial  [2]:instrumental >> ", >> logfn )

  c = scan( buf )
  while( buf != "1" && buf != "2" )
    c = scan( buf )
  printf( "%s\n", buf, >> logfn )
  if( buf == "1" )
  {
    flag_mode = 0
    printf( "Entering 'equatorial coodinate system' mode...\n" )
    printf( "Entering 'equatorial coodinate system' mode...\n", >> logfn )
  }
  else
  {
    flag_mode = 1
    printf( "Entering 'instrumental coodinate system' mode...\n" )
    printf( "Entering 'instrumental coodinate system' mode...\n", >> logfn )
  }
  if( access( obslog ) == 1 )
    delete( obslog, ver- )

#----- check old list

  if( access( grouplist ) == 0 )
  {
    printf( "Cannot find %s. Please do 'hbsiquave'\n", grouplist )
    printf( "Cannot find %s. Please do 'hbsiquave'\n", grouplist, >> logfn )
    bye
  }

#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  else
    {
      n_version = 211
    }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n" )
    printf( "cl> set version=V2.10  (V2.10 and V2.11 are valid)\n" )
    printf( "IRAF version check failed.\n", >> logfn )
    printf( "Please input like next line, and restart 'hbsred'\n", >> logfn )
    printf( "cl> set version=V2.10  (V2.10 and V2.11 are valid)\n", >> logfn )
    bye
  }

#----- scanning list file

  n_list = 0

  list = grouplist

  while( fscan( list, fn_head, diaph, binfile, buf, buf, buf, fn_cmnt ) != EOF )
  {
    n_list += 1
    fn_head0[n_list] = fn_head
    diaph0[n_list] = diaph
    binfile0[n_list] = binfile
    fn_cmnt0[n_list] = fn_cmnt
  }

#----- main routine

  for( i=1; i<=n_list; i+=1 )
  {

    fn_head =  fn_head0[i]
    diaph = diaph0[i]
    binfile = binfile0[i]
    fn_cmnt = fn_cmnt0[i]
  
nextstep1:
  
    flag_i = 0
    flag_q = 0
    flag_u = 0

#----- assign filename

    fn_i1ave = fn_head//"i1.ave"
    fn_i1sig = fn_head//"i1.sig"
    fn_i2ave = fn_head//"i2.ave"
    fn_i2sig = fn_head//"i2.sig"
    fn_qave = fn_head//"q.ave"
    fn_qsig = fn_head//"q.sig"
    fn_uave = fn_head//"u.ave"
    fn_usig = fn_head//"u.sig"
    fn_k0ave = fn_head//"k0.ave"
    fn_k1ave = fn_head//"k1.ave"
    fn_k2ave = fn_head//"k2.ave"
    fn_k3ave = fn_head//"k3.ave"
    fn_kn = fn_head//"k.num"

    if( flag_mode == 0 )
    {
      fn_out1 = fn_head//"0.xy"
      fn_out2 = fn_head//"1.xy"
      if( substr( diaph, 4, 4 ) == "L" )
      {
        upfile = rdir//uphead//diaph
        gtfile = rdir//gthead//diaph
        spfile = rdir//sphead//diaph
      }
      else
      {
        upfile = rdir//uphead//diaph
        gtfile = rdir//gthead//diaph
        spfile = rdir//sphead//diaph
      }
    }
    else
    {
      fn_out1 = fn_head//"0.ins"
      fn_out2 = fn_head//"1.ins"
      upfile = "tmp"
      gtfile = "tmp"
      spfile = "tmp"
    }


#----- check images

    if( access( fn_i1ave//".imh" ) == 0 )
    {
#      printf( "Warning: cannot find '%s'\n", fn_i1ave )
      flag_i = -1
    }
    else
    {
      imgets( fn_i1ave, "JD" )
      jd = real( imgets.value )
      imgets( fn_i1ave, "AIRMASS" )
      am = real( imgets.value )
      imgets( fn_i1ave, "EXPTIME" )
      exptime = real( imgets.value )
      imgets( fn_i1ave, "N_AVE" )
      n_i = int( imgets.value )
    }

    if( access( fn_i1sig//".imh" ) == 0 )
    {
#      printf( "Warning: cannot find '%s'\n", fn_i1sig )
      flag_i = -1
    }
    if( access( fn_i2ave//".imh" ) == 0 )
    {
#      printf( "Warning: cannot find '%s'\n", fn_i2ave )
      flag_i = -1
    }
    if( access( fn_i2sig//".imh" ) == 0 )
    {
#      printf( "Warning: cannot find '%s'\n", fn_i2sig )
      flag_i = -1
    }

    n_q = 0
    if( access( fn_qave//".imh" ) == 0 )
    {
#      printf( "Warning: cannot find '%s'\n", fn_qave )
      flag_q = -1
    }
    if( access( fn_k0ave//".imh" ) == 1 )
    {
      imgets( fn_k0ave, "N_AVE" )
      n_q += int( imgets.value )
    }
    if( access( fn_k2ave//".imh" ) == 1 )
    {
      imgets( fn_k2ave, "N_AVE" )
      n_q += int( imgets.value )
    }
    if( access( fn_qsig//".imh" ) == 0 )
    {
#      printf( "Warning: cannot find '%s'\n", fn_qsig )
      flag_q = -1
    }

    n_u = 0
    if( access( fn_uave//".imh" ) == 0 )
    {
#      printf( "Warning: cannot find '%s'\n", fn_uave )
      flag_u = -1
    }
    if( access( fn_k1ave//".imh" ) == 1 )
    {
      imgets( fn_k1ave, "N_AVE" )
      n_u += int( imgets.value )
    }
    if( access( fn_k3ave//".imh" ) == 1 )
    {
      imgets( fn_k3ave, "N_AVE" )
      n_u += int( imgets.value )
    }
    if( access( fn_usig//".imh" ) == 0 )
    {
#      printf( "Warning: cannot find '%s'\n", fn_usig )
      flag_u = -1
    }

    if( access( fn_i1ave ) )
      delete( fn_i1ave, ver- )
    if( access( fn_i1sig ) )
      delete( fn_i1sig, ver- )
    if( access( fn_i2ave ) )
      delete( fn_i2ave, ver- )
    if( access( fn_i2sig ) )
      delete( fn_i2sig, ver- )
    if( access( fn_qave ) )
      delete( fn_qave, ver- )
    if( access( fn_qsig ) )
      delete( fn_qsig, ver- )
    if( access( fn_uave ) )
      delete( fn_uave, ver- )
    if( access( fn_usig ) )
      delete( fn_usig, ver- )

#----- search calibration file ( if celetial mode )

    if( flag_mode == 0 )
    {
      if( access( upfile ) == 0 )
      {
        printf( "Cannot find '%s' [s]:skip [e]:instrumental mode >> ",\
	        upfile )
        printf( "Cannot find '%s' [s]:skip [e]:instrumental mode >> ",\
	        upfile, >> logfn )
        c = scan( buf )
	while( buf != "s" && buf != "e" )
          c = scan( buf )
        printf( "%s\n", buf, >> logfn )
	if( buf == "s" )
	  next
	flag_mode = 1
	goto nextstep1
      }
      if( access( gtfile ) == 0 )
      {
        printf( "Cannot find '%s' [s]:skip [e]:instrumental mode >> ",\
	        gtfile )
        printf( "Cannot find '%s' [s]:skip [e]:instrumental mode >> ",\
	        gtfile, >> logfn )
        c = scan( buf )
	while( buf != "s" && buf != "e" )
          c = scan( buf )
        printf( "%s\n", buf, >> logfn )
	if( buf == "s" )
	  next
	flag_mode = 1
	goto nextstep1
      }
      if( access( spfile ) == 0 )
      {
        printf( "Cannot find '%s' [s]:skip [e]:instrumental mode >> ",\
	        spfile )
        printf( "Cannot find '%s' [s]:skip [e]:instrumental mode >> ",\
	        spfile, >> logfn )
        c = scan( buf )
	while( buf != "s" && buf != "e" )
          c = scan( buf )
        printf( "%s\n", buf, >> logfn )
	if( buf == "s" )
	  next
	flag_mode = 1
	goto nextstep1
      }
    }


#----- display

    printf( "%s --> %s %s ", fn_head, fn_out1, fn_out2 )
    printf( "%s --> %s %s ", fn_head, fn_out1, fn_out2, >> logfn )
    if( flag_mode == 0 )
    {
      printf( "(equatorial coordinate)\n" )
      printf( "(equatorial coordinate)\n", >> logfn )
    }
    else
    {
      printf( "(instrumental coordinate)\n" )
      printf( "(instrumental coordinate)\n", >> logfn )
    }

#----- calling c program

    if( n_version == 210 )
    {
      if( flag_i == 0 )
      {
        wfits( fn_i1ave//".imh", fn_i1ave, bscale=1.0, bzero=0.0,\
	       long_header-, short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fn_i1sig//".imh", fn_i1sig, bscale=1.0, bzero=0.0,\
               long_header-, short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fn_i2ave//".imh", fn_i2ave, bscale=1.0, bzero=0.0,\
	       long_header-, short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fn_i2sig//".imh", fn_i2sig, bscale=1.0, bzero=0.0,\
	       long_header-, short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fn_kn//".imh", fn_kn, bscale=1.0, bzero=0.0,\
	       long_header-, short_header-, bitpix=-32, scale-, autoscale- )
      }
      if( flag_q == 0 )
      {
        wfits( fn_qave//".imh", fn_qave, bscale=1.0, bzero=0.0, long_header-,\
             short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fn_qsig//".imh", fn_qsig, bscale=1.0, bzero=0.0, long_header-,\
             short_header-, bitpix=-32, scale-, autoscale- )
      }
      if( flag_u == 0 )
      {
        wfits( fn_uave//".imh", fn_uave, bscale=1.0, bzero=0.0, long_header-,\
             short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fn_usig//".imh", fn_usig, bscale=1.0, bzero=0.0, long_header-,\
             short_header-, bitpix=-32, scale-, autoscale- )
      }
    }
    else
    {
      if( flag_i == 0 )
      {
        wfits( fn_i1ave//".imh", fn_i1ave, bscale=1.0, bzero=0.0, long_header-,
               fextn="", short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fn_i1sig//".imh", fn_i1sig, bscale=1.0, bzero=0.0, long_header-,
               fextn="", short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fn_i2ave//".imh", fn_i2ave, bscale=1.0, bzero=0.0, long_header-,
               fextn="", short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fn_i2sig//".imh", fn_i2sig, bscale=1.0, bzero=0.0, long_header-,
               fextn="", short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fn_kn//".imh", fn_kn, bscale=1.0, bzero=0.0, long_header-,
               fextn="", short_header-, bitpix=-32, scale-, autoscale- )
      }
      if( flag_q == 0 )
      {
        wfits( fn_qave//".imh", fn_qave, bscale=1.0, bzero=0.0, long_header-,
               fextn="", short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fn_qsig//".imh", fn_qsig, bscale=1.0, bzero=0.0, long_header-,
               fextn="", short_header-, bitpix=-32, scale-, autoscale- )
      }
      if( flag_u == 0 )
      {
        wfits( fn_uave//".imh", fn_uave, bscale=1.0, bzero=0.0, long_header-,
               fextn="", short_header-, bitpix=-32, scale-, autoscale- )
        wfits( fn_usig//".imh", fn_usig, bscale=1.0, bzero=0.0, long_header-,
               fextn="", short_header-, bitpix=-32, scale-, autoscale- )
      }
    }

    if( access( tmp_iqu ) == 1 )
      delete( tmp_iqu, ver- )
    printf( "%s\n", fn_kn, >> tmp_iqu )
    printf( "%s %s %d\n", fn_i2ave, fn_i2sig, flag_i, >> tmp_iqu )
    printf( "%s %s %d\n", fn_qave, fn_qsig, -1, >> tmp_iqu )
    printf( "%s %s %d\n", fn_uave, fn_usig, -1, >> tmp_iqu )
    printf( "%d\n", flag_mode, >> tmp_iqu )
    printf( "%s\n", upfile, >> tmp_iqu )
    printf( "%s\n", gtfile, >> tmp_iqu )
    printf( "%s\n", spfile, >> tmp_iqu )
    hbsiqupt( tmp_iqu, "NONE", fnphot, "NONE", etcdir, fn_out2 )

    if( access( tmp_iqu ) == 1 )
      delete( tmp_iqu, ver- )
    printf( "%s\n", fn_kn, >> tmp_iqu )
    printf( "%s %s %d\n", fn_i1ave, fn_i1sig, flag_i, >> tmp_iqu )
    printf( "%s %s %d\n", fn_qave, fn_qsig, flag_q, >> tmp_iqu )
    printf( "%s %s %d\n", fn_uave, fn_usig, flag_u, >> tmp_iqu )
    printf( "%d\n", flag_mode, >> tmp_iqu )
    printf( "%s\n", upfile, >> tmp_iqu )
    printf( "%s\n", gtfile, >> tmp_iqu )
    printf( "%s\n", spfile, >> tmp_iqu )
    hbsiqupt( tmp_iqu, binfile, fnphot, fn_out2, etcdir, fn_out1 )

    #--- If comment file exists...

    cmnt = ""
    if( access( fn_cmnt ) == 1 )
    {
      list = fn_cmnt
      if( fscan( list, buf ) == 1 )
        cmnt = buf
    }

    #--- Output averaging log

    printf( "%13s %s %13.5f %7.4f %5.1f %2d %2d %s\n", fn_out1,\
            diaph, jd, am, exptime, n_q, n_u, cmnt, >> obslog ) 

    if( access( fn_i1ave ) == 1 )
      delete( fn_i1ave, ver- )
    if( access( fn_i1sig ) == 1 )
      delete( fn_i1sig, ver- )
    if( access( fn_i2ave ) == 1 )
      delete( fn_i2ave, ver- )
    if( access( fn_i2sig ) == 1 )
      delete( fn_i2sig, ver- )
    if( access( fn_qave ) == 1 )
      delete( fn_qave, ver- )
    if( access( fn_qsig ) == 1 )
      delete( fn_qsig, ver- )
    if( access( fn_uave ) == 1 )
      delete( fn_uave, ver- )
    if( access( fn_usig ) == 1 )
      delete( fn_usig, ver- )
    if( access( fn_kn ) == 1 )
      delete( fn_kn, ver- )
  }
  printf( "\n*** summary (file: %s)***\n", obslog )
  type( obslog )
  printf( "\n*** summary (file: %s)***\n", obslog, >> logfn )
  type( obslog, >> logfn )

end


