#
#       HBS REDUCTION SUB ROUTINE
#
#        Make table files of photometoric parameters : hbsphfit3.cl
#
#       1998/10/22      K.Kawabata  Ver.2.00
#       2000-02-20      K.Kawabata  Ver.3.00
#       2007/09/13      M.Isogai    Ver.4.00 for AndorCCD 
#

procedure hbsphfit3

begin

  file infn1, infn2, infn3, outfn

  real conv, extn, norm
  real thro, effconv
  real geadu
  int pregain, postgain

  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7, buf8
  string tmp_fn
  real cdelt, crval, crpix
  int npix
  file caldir, fn_def

  int c, i
  real lmd

#--- initial settings

  caldir = envget( "_caldir" )
  fn_def = caldir//envget( "_defhname" )

  tmp_fn = mktemp( "hbs.tmp.fn." )

  infn1 = "HBS.phot.convf"
  infn2 = "HBS.phot.extn"
  infn3 = "HBS.phot.refspec"
  outfn = envget( "_photmet" )

  print( "" )
  print( "------------------------------" )
  print( " Photmetric Parameter Output"
  print( "------------------------------" )
  print( "" )


#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    bye
  }

  crpix = 1.0
  crval = 0.0
  cdelt = 0.0

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "SW" )
    {
      crval = real( buf3 )
   }
    if( buf2 == "DW" )
    {
      cdelt = real( buf3 )
    }
    if( buf2 == "NX3" )
    {
      npix = int( buf3 )
    }
    if( buf2 == "GEADU" )
    {
      geadu = real( buf3 )
    }
  }
  if( ( crval * cdelt ) == 0 )
  {
    printf( "Read error in '%s'\n", fn_def )
    bye
  }

#--- check input images

  if( access( infn1//".imh" ) == 0 )
  {
    printf( "'%s' not found.\n", infn1 )
    bye
  }
  if( access( infn2//".imh" ) == 0 )
  {
    printf( "'%s' not found.\n", infn2 )
    bye
  }
  if( access( infn3//".imh" ) == 0 )
  {
    printf( "'%s' not found.\n", infn3 )
    bye
  }

#--- delete old output file

  if( access( outfn ) == 1 )
    delete( outfn, ver- )

  for( i=1; i<=npix; i+=1 )
  {
#    if( i ! 100 = 0 )
#      printf( "%d ", i )
    lmd = (i-1)*cdel + crval

  #--- read input images

    imgets( infn1, "DETECTOR" )
    buf1 = imgets.value

      if( buf1 == "1024x1024 TI CCD" || buf1 == "T5 CRYOCAM (TC215)" \
	|| buf1 == "S5 CRYOCAM (SITe)" )
	{
	  imgets( infn1, "PREGAIN" )
          pregain = int( imgets.value )
          imgets( infn1, "POSTGAIN" )
          postgain = int( imgets.value )
	}
      else if( buf1 == "Andor DV440" )
        {
	  pregain = 1.
	  postgain = 1.
        }
      else
        {
          printf( "Header keyword 'DETECTOR' wrong (%s)\n", imgets.value )
          printf( "TI CCD is assumed.\n" )
          printf( "Header keyword 'DETECTOR' wrong (%s)\n", imgets.value,\
                   >> logfn )
          printf( "TI CCD is assumed.\n", >> logfn )

	  imgets( infn1, "PREGAIN" )
          pregain = int( imgets.value )
          imgets( infn1, "POSTGAIN" )
          postgain = int( imgets.value )

         } 


    imstatistics( infn1//"["//i//":"//i//",*]",\
                  format-, fields="mean", >> tmp_fn )
    list = tmp_fn
    if( fscan( list, conv ) != 1 )
    {
      printf( "Imstatistics-file read error for %s for x=%d pix\n", infn1, i )
      bye
    }
    delete( tmp_fn, ver- )

    imstatistics( infn2//"["//i//":"//i//",*]",\
                  format-, fields="mean", >> tmp_fn )
    list = tmp_fn
    if( fscan( list, extn ) != 1 )
    {
      printf( "Imstatistics-file read error for %s for x=%d pix\n", infn2, i )
      bye
    }
    delete( tmp_fn, ver- )

    imstatistics( infn3//"["//i//":"//i//",*]",\
                  format-, fields="mean", >> tmp_fn )
    list = tmp_fn
    if( fscan( list, norm ) != 1 )
    {
      printf( "Imstatistics-file read error for %s for x=%d pix\n", infn3, i )
      bye
    }
    delete( tmp_fn, ver- )

    conv = conv * ( geadu * pregain * postgain )

    effconv = conv / norm
    thro = 6.626e-27 / ( effconv )

    printf( "%7.2f %10e %10e %10e  %10e %10e\n", lmd, conv, extn,\
            norm, effconv, thro, >> outfn )
  }

  printf( "Photometric parameter file: %s created.\n", outfn )

end
