#######################################################
#               HBS REDUCTION on IRAF                 #
#                                                     #
#   main program for spectrophotometry: hbsphred.cl   #
#                                                     #
#            2000/02/20  Ver.3.00   K.Kawabata        #
#######################################################

procedure hbsphred

begin

#----- const variables definision

  file histfile = "HBS.phot.history"
  string procname[10]
  int n_proc

#----- volatile variables definition

  int procid
  int procid0
  int procid1
  int c, last_c, i
  string buf = ""
  string buf1 = ""
  string buf2 = ""
  string buf3 = ""
  int flag_hist = 0

#----- constant variable definitions

  procname[1] = "hbsphgroup"
  procname[2] = "hbsphfit1"
  procname[3] = "hbsphfit2"
  procname[4] = "hbsphfit3"
  procname[5] = "hbsphfit4"
  procname[6] = "hbsphfit5"
  procname[7] = "hbsphcor"
  n_proc = 7

  procid = 1
  procid0 = 1
  procid1 = n_proc

#----- DELETE OLD TEMPORALY FILES

  printf( "Now clearing tempolary files...  Just a minute.\n" )
  imdelete('*tmp*.imh')
  delete('*tmp*')

#----- TITLE

  printf( "\n" )
  printf( "{~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}\n" )
  printf( "{  HBS SPECTROPHOTMETRIC REDUCTION Ver 3.00 (Feb-29-2000)  }\n" )
  printf( "{                                                          }\n" )
  printf( " ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"  )
  printf( "\n" )

#----- CHECK INITIALIZATION SCRIPT

  if( envget( "_initproc" ) != "DONE" )
  {
    printf( "Error: Not Done Initialization.\n" )
    printf( "Please do '%s'\n", initproc )
    bye
  }


#---- Read history file

  if( access( histfile ) == 1 )
  {
    printf( "\nHistory file detected.\n" )
    list = histfile
    c = fscan( list, buf1, procid0, buf2, buf3 )
    last_c = c
    while( c != EOF )
    {
      last_c = c
      c = fscan( list, buf1, procid0, buf2, buf3 )
    }
    if( last_c < 4 )
    {
      buf3 = "ABNORMAL_TERMINATION"
      printf( "%s\n", buf3, >> histfile )
    }

    if( procid0 < 1 || procid0 > n_proc )
    {
      printf( "Error: ProcID is wrong in %s. Ignored.\n", histfile )
    }
    else
    {
      flag_hist = 1
      printf( "LAST PROCEDURE NAME : %s (procID %d)\n",\
              procname[procid0], procid0 )
      printf( "               DATE : %s\n", buf1 )
      if( buf3 == "NORMAL_END" )
      {
        printf( "             STATUS : NORMAL END\n" )
        procid = procid0 + 1
      }
      else
      {
        printf( "             STATUS : ABNORMAL TERMINATION\n" )
        procid = procid0
      }
    }
  }

#----- Select Procedure

  if( flag_hist == 1 )
    printf( "[RET]:Continue.(from '%s') [1]:Customize >> ",\
            procname[procid] )
  else
    printf( "[RET]:New Reduction [1]:Customize >> " )
  buf = ""
  c = scan( buf )
  while( buf != "" && buf != "1" )
  {
    buf = ""
    c = scan( buf )
  }
  if( buf == "1" )
  {
    for( i=1; i<=n_proc; i+=1 )
      printf( "%2d  %s\n", i, procname[i] )
    printf( "Biginning Proc-ID number >> " )
    c = scan( procid )
    while( procid < 0 || procid > n_proc )
      c = scan( procid )
    printf( "End Proc-ID number >> " )
    c = scan( procid1 )
    while( procid1 < 0 || procid1 > n_proc )
      c = scan( procid1 )
  }

#----- main routine

  while( procid > 0 && procid <= procid1 )
  {
    if( access( "hbs.tmp.time" ) == 1 )
      delete( "hbs.tmp.time", ver- )
    time >> hbs.tmp.time

    list = "hbs.tmp.time"
    if( fscan( list, buf, buf1, buf2 ) != 3 )
    {
      printf( "Error: time task/read\n" )
      bye
    }
    delete( "hbs.tmp.time", ver- )
    printf( "\n%s-%s  %2d %s\n", buf2, buf1, procid, procname[procid] )
    printf( "%s-%s  %2d %s  ", buf2, buf1, procid, procname[procid],\
            >> histfile )
    
    switch( procid )
    {
      case 1:
        hbsphgroup
        break
      case 2:
        hbsphfit1
        break
      case 3:
        hbsphfit2
        break
      case 4:
        hbsphfit3
        break
      case 5:
        hbsphfit4
        break
      case 6:
        hbsphfit5
        break
      case 7:
        hbsphcor
        break
      default:
        printf( "Procedure ID error !!!  Restart,please.\n" )
        printf( "(If you want re-examine, select manual mode)\n" )
	bye
    }
    printf( "NORMAL_END\n", >> histfile )
    procid += 1

  }

#----- ending

  if( procid > n_proc )
  {
    printf( "Minimize disk space? (y/n) >> " )
    buf = ""
    c = scan( buf )
    while( buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "y" )
      hbsclean
  }

end

