#
#       HBS REDUCTION SUB ROUTINE
#
#        photometric error estimation : hbsphsig.cl
#
#       1998/07/31      K.Kawabata  Ver.2.00
#       1998/09/28      K.Kawabata  Ver.2.10
#       2005/04/07      M.Isogai    Ver.2.20 for iraf v2.12
#       2008/11/19      M.Isogai    Ver.2.30 for iraf v2.14
#

procedure hbsphsig( cnt_image, ref_bin, cmpave_image, cmpsig_image, outimage )

file cnt_image { prompt = "Input image of un-binned count" }
file ref_bin { prompt = "Binning factor table" }
file cmpave_image { prompt = "Input image of comparison's average count" }
file cmpsig_image { prompt = "Input image of comparison's sigma" }
file outimage { prompt = "Output image of error" }

begin

  file fncnt, fnin, fncmpave, fncmpsig, fnout
  file reffn, refdir, fn_def
  real fac
  string tmp_fn1, tmp_fn2, tmp_fn3, tmp_fn4
  int binf = 7
  string buf1, buf2, buf3
  int n_version      # IRAF version (210 or 211)
  int c
  int n_ci
  real geadu, preg, postg

#--- initial settings

  refdir = envget( "_etcdir" )
  fn_def = envget( "_caldir" )//envget( "_defhname" )

  tmp_fn1 = mktemp( "hbs.tmp.fn." )
  tmp_fn2 = mktemp( "hbs.tmp.fn." )
  tmp_fn3 = mktemp( "hbs.tmp.fn." )
  tmp_fn4 = mktemp( "hbs.tmp.fn." )

  fnin = cnt_image
  fncnt = "hbs.tmp.bincount"
  reffn = ref_bin
  fncmpave = cmpave_image
  fncmpsig = cmpsig_image
  fnout = outimage

  if( access( fncnt//".imh" ) == 1 )
    imdelete( fncnt, ver- )

#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  else
    {
      n_version = 211
    }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n"
    printf( "cl> set version=V2.10  (V2.10 and V2.11 are valid)\n"
    bye
  }


#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    bye
  }

  geadu = 0.0

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "GEADU" )
    {
      geadu = real( buf3 )
    }
  }
  if( geadu == 0 )
  {
    printf( "Read error in '%s'\n", fn_def )
    bye
  }

#----

  imgets( fnin, "PREGAIN" )
  preg = int( imgets.value )
  imgets( fnin, "POSTGAIN" )
  postg = int( imgets.value )

  imgets( fncmpave, "N_AVE" )
  n_ci = int( imgets.value )

  fac = geadu * preg * postg
  printf( "ADUperELECTRON = %4.2f\n", fac )

#  boxcar( fncnt, fncnt, binf, 1 )
#  boxcar( fncmpave, fncmpave, binf, 1 )
#  boxcar( fncmpsig, fncmpsig, binf, 1 )

  if( n_version == 210 )
  { 
    wfits( fnin//".imh", tmp_fn1, bscale=1.0, bzero=0.0, long_header-,
           short_header-, bitpix=-32, scale-, autoscale- )
  }
  else
  {
    wfits( fnin//".imh", tmp_fn1, bscale=1.0, bzero=0.0, long_header-,
           fextn="", short_header-, bitpix=-32, scale-, autoscale- )
  }
  copy( tmp_fn1, tmp_fn2, ver- )
  hbscbin( tmp_fn1, reffn, refdir, "NONE", tmp_fn2, "1" )

  if( n_version == 210 )
  {
    rfits( tmp_fn2, "", fncnt, make_image+, long_header-, short_header-,\
           datatype="real", blank=0., scale-, oldirafname-, offset=0 )
  }
  if( n_version == 211 )
  {
    rfits( tmp_fn2, "", fncnt//".imh", make_image+, long_header-,\
           short_header-,\
           datatype="real", blank=0., scale-, oldirafname-, offset=0 )
  }

  delete( tmp_fn1, ver- )
  delete( tmp_fn2, ver- )

  imarith( fncnt, "/", fac, tmp_fn1, ver- )
  imfunc( tmp_fn1, tmp_fn2, "sqrt", ver- )
  imarith( tmp_fn2, "/", tmp_fn1, tmp_fn1, ver- )
  imdelete( tmp_fn2, ver- )

  imarith( fncmpsig, "/", sqrt( n_ci ), tmp_fn4, ver- )
  imarith( tmp_fn4, "/", fncmpave, tmp_fn2, ver- )
  imdelete( tmp_fn4, ver- )

  imarith( tmp_fn1, "*", tmp_fn1, tmp_fn1, ver- )
  imarith( tmp_fn2, "*", tmp_fn2, tmp_fn2, ver- )
  imarith( tmp_fn1, "+", tmp_fn2, tmp_fn3, ver- )

  if( access( fnout//".imh" ) == 1 )
    imdelete( fnout, ver- )
  imfunc( tmp_fn3, fnout, "sqrt", ver- )

  imdelete( tmp_fn1, ver- )
  imdelete( tmp_fn2, ver- )
  imdelete( tmp_fn3, ver- )

end
