#
#       HBS REDUCTION SUB ROUTINE
#
#      Scaling and modify count distribution for x direction
#                                    (using data made by hbs1to1y.cl)
#                                                 : hbsscxsub.cl
#
#       2005/04/07   Modified by M.Isogai    for iraf v2.12
#       2008/11/19   Modified by M.Isogai    for iraf v2.14
#

procedure hbsscx( infilee, infileo, ref_dir, extfile, ordfile )

file infilee { prompt = "Input extraordinary image" }
file infileo { prompt = "Input ordinary image" }
string ref_dir { prompt = "Directry of fitted coef. table" }
file extfile { prompt = "Output image of extraordinary ray" }
file ordfile { prompt = "Output image of ordinary ray" }

begin

  file infne, infno, outfne, outfno
  file fn_fite, fn_fito, fn_lmd
  file fn_fite1, fn_fito1, fn_lmd1
  string refdir
  int i, j, k, n_list, c
  int nx, ny
  string buf = ""
  real c0[40], c1[40], c2[40], c3[40], c4[40], c5[40]
  string buf1, buf2, buf3
  int n_version      # IRAF version (210 or 211)

#----

  fn_fite1 = envget( "_1to1se" )
  fn_fito1 = envget( "_1to1so" )
  fn_lmd1 = envget( "_1to1sw" )

  infne = infilee
  infno = infileo
  refdir = ref_dir
  outfne = extfile
  outfno = ordfile

  fn_fite = refdir//fn_fite1
  fn_fito = refdir//fn_fito1
  fn_lmd = refdir//fn_lmd1


#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  else
    {
      n_version = 211
    }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n"
    printf( "cl> set version=V2.10  (V2.10 and V2.11 are valid)\n"
    bye
  }

#---

  if( access( fn_fite ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_fite )
    bye
  }
  if( access( fn_fito ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_fito )
    bye
  }

  if( access( infne//".imh" ) == 0 )
  {
    printf( "Cannot find '%s.imh'\n", infne )
    bye
  }
  if( access( infno//".imh" ) == 0 )
  {
    printf( "Cannot find '%s.imh'\n", infno )
    bye
  }

#---- make FITS file

  if( access( infne ) == 1 )
    delete( infne, ver- )
  if( access( infno ) == 1 )
    delete( infno, ver- )

  if( n_version == 210 )
  {
    wfits( infne//".imh", infne, bscale=1.0, bzero=0.0, long_header-,
           short_header-, bitpix=-32, scale-, autoscale- )
    wfits( infno//".imh", infno, bscale=1.0, bzero=0.0, long_header-,
            short_header-, bitpix=-32, scale-, autoscale- )
  }
  else
  {
    if( n_version == 211 )
    {
      wfits( infne//".imh", infne, bscale=1.0, bzero=0.0, long_header-,
             fextn="", short_header-, bitpix=-32, scale-, autoscale- )
      wfits( infno//".imh", infno, bscale=1.0, bzero=0.0, long_header-,
             fextn="", short_header-, bitpix=-32, scale-, autoscale- )
    }
    else
    {
      printf( "IRAF version erorr (%d)\n", n_version )
      bye
    }
  }

  copy( infne, outfne, ver- )
  copy( infno, outfno, ver- )

#---- transform

printf( "e " )
#sleep( 1 )
wait
  hbstfx( infne, fn_fite, fn_lmd, outfne )
printf( "o " )
#sleep( 1 )
wait
  hbstfx( infno, fn_fito, fn_lmd, outfno )

  delete( infne, ver- )
  delete( infno, ver- )

#---- conversion to iraf format

  if( access( outfne//".imh" ) == 1 )
    imdelete( outfne, ver- )
  if( access( outfno//".imh" ) == 1 )
    imdelete( outfno, ver- )


  if( n_version == 210 )
  {
    rfits( outfne, "*", outfne, make_image+, long_header-, short_header-,\
           datatype="real", blank=0., scale-, oldirafname-, offset=0 )
    rfits( outfno, "*", outfno, make_image+, long_header-, short_header-,\
           datatype="real", blank=0., scale-, oldirafname-, offset=0 )
  }
  else
  {
    if( n_version == 211 )
    {
      rfits( outfne, "", outfne//".imh", make_image+, long_header-,\
             short_header-,\
             datatype="real", blank=0., scale-, oldirafname-, offset=0 )
      rfits( outfno, "", outfno//".imh", make_image+, long_header-,\
             short_header-,\
             datatype="real", blank=0., scale-, oldirafname-, offset=0 )
    }
    else
    {
      printf( "IRAF version erorr (%d)\n", n_version )
      bye
    }
  }

  delete( outfne, ver- )
  delete( outfno, ver- )
  
end
