#
#       HBS REDUCTION SUB ROUTINE
#
#      Scaling and modify count distribution for y direction
#                                    (using data made by hbs1to1x.cl)
#                                                 : hbsscysub.cl
#
#       2005/04/07   Modified by M.Isogai    for iraf v2.12
#       2008/11/19   Modified by M.Isogai    for iraf v2.14
#

procedure hbsscy( infile, ref_dir, extfile, ordfile )

file infile { prompt = "Input image" }
string ref_dir { prompt = "Directry of fitted coef. table" }
file extfile { prompt = "Output image of extraordinary ray" }
file ordfile { prompt = "Output image of ordinary ray" }

begin

  file infn, outfne, outfno
  file fn_fite, fn_fito
  file fn_fite1 = "distorth.ext.tbl"
  file fn_fito1 = "distorth.ord.tbl"
  string refdir
  int i, j, k, n_list, c
  int nx, ny
  string buf = ""
  real c0[40], c1[40], c2[40], c3[40], c4[40], c5[40]
  int n_feature = 21
  string buf1, buf2, buf3
  int n_version      # IRAF version (210 or 211)

#----

  infn = infile
  refdir = ref_dir
  outfne = extfile
  outfno = ordfile

  fn_fite1 = envget( "_1to1we" )
  fn_fito1 = envget( "_1to1wo" )

  fn_fite = refdir//fn_fite1
  fn_fito = refdir//fn_fito1


#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  else
    {
      n_version = 211
    }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n"
    printf( "cl> set version=V2.10  (V2.10 and V2.11 are valid)\n"
    bye
  }

#---

  if( access( fn_fite ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_fite )
    bye
  }
  if( access( fn_fito ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_fito )
    bye
  }

  if( access( infn//".imh" ) == 0 )
  {
    printf( "Cannot find '%s.imh'\n", infn )
    bye
  }

#---- make FITS file

  if( access( infn ) == 1 )
    delete( infn, ver- )

  if( n_version == 210 )
  {
    wfits( infn, infn, bscale=1.0, bzero=0.0, long_header-,
           short_header-, bitpix=-32, scale-, autoscale- )
  }
  else
  {
    if( n_version == 211 )
    {
      wfits( infn//".imh", infn, bscale=1.0, bzero=0.0, long_header-,
             fextn="", short_header-, bitpix=-32, scale-, autoscale- )
    }
    else
    {
      printf( "IRAF version erorr (%d)\n", n_version )
      bye
    }
  }

  copy( infn, outfne )
  copy( infn, outfno )

#---- transform

  hbstfy( infn, fn_fite, outfne )
  hbstfy( infn, fn_fito, outfno )

  delete( infn, ver- )

#---- conversion to iraf format

  if( access( outfne//".imh" ) == 1 )
    imdelete( outfne, ver- )
  if( access( outfno//".imh" ) == 1 )
    imdelete( outfno, ver- )


  if( n_version == 210 )
  {
    rfits( outfne, "*", outfne, make_image+, long_header-, short_header-,\
           datatype="real", blank=0., scale-, oldirafname-, offset=0 )
    rfits( outfno, "*", outfno, make_image+, long_header-, short_header-,\
           datatype="real", blank=0., scale-, oldirafname-, offset=0 )
  }
  else
  {
    if( n_version == 211 )
    {
      rfits( outfne, "", outfne//".imh", make_image+, long_header-,\
             short_header-,\
             datatype="real", blank=0., scale-, oldirafname-, offset=0 )
      rfits( outfno, "", outfno//".imh", make_image+, long_header-,\
             short_header-,\
             datatype="real", blank=0., scale-, oldirafname-, offset=0 )
    }
    else
    {
      printf( "IRAF version erorr (%d)\n", n_version )
      bye
    }
  }


  delete( outfne, ver- )
  delete( outfno, ver- )
  
end
