/***********************************************************

    hbs reduction subroutine: hbstfy.c

                                   1998/01/10  K. Kawabata
                                   2005/09/06  M. Isogai     Andor CCD

     transformation of image in order to correct distortions


    usage: % hbstfy infile ref_table outfile

      infile -- input image file (FITS format)

      ref_table -- fitted coeficients table (made by hbs1to1x.cl)

      outfile -- output spectrum file (FITS format)

************************************************************/
		  
#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<math.h>
#include"fitsio.h"

#include"hbsdefine.h"

#define FITORDER 6 /* fitting order in hbs1to1x.cl */

#ifndef NCCD
#define NCCD 1     /* CCD chip number 1:TC215 2:SITe */
#endif

int readpeakfit( char *infile, double coef[][FITORDER]  ){

  FILE *fp;
  int n=0;
  char buf[100], last_buf[100]="";

  if( ( fp = fopen( infile, "rt" ) ) == NULL ){
    return( -1 );
  }

  while( feof( fp ) == 0 ){
    fgets( buf, 100, fp );
    if( strcmp( buf, last_buf ) == 0 )
      continue;
    sscanf( buf, "%lf %lf %lf %lf %lf %lf", &coef[n][0], &coef[n][1],\
	    &coef[n][2], &coef[n][3], &coef[n][4], &coef[n][5] );
    n++;
    strcpy( last_buf, buf );
  }
  fclose( fp );
  return( n );
}

int scanfile( char *infile, float count[][NX1]  ){

  fitsfile *fp;
  int i, anynul, status;
  char buf[100];
  float nulval = 0.;
  long naxis1, naxis2, npix;
  char comment[100];

  status = 0;

  if( fits_open_file( &fp, infile, READONLY, &status ) != 0 ){
    printf( "cannot open file '%s' status=%d\n", infile, status );
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS1", &naxis1,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS2", &naxis2,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( naxis1 != NX1 || naxis2 != NY1 ){
    printf( "error: definition of NX1 or NY1 in 'hbsdefine.h'\n" );
    return( -1 );
  }
  npix = naxis1 * naxis2;

  if( fits_read_img( fp, TFLOAT, 1L, npix, &nulval,\
      count[0], &anynul, &status ) != 0 ){
    printf( "cannot read from '%s'\n", infile );
    return( status );
  }
  if( fits_close_file( fp, &status ) != 0 ){
    return( status );
  }

  return( 0 );

}

double realcount1( float c[][NX1], int x, double y_low, double y_high ){

  int iy_low, iy_high;
  int i, j;
  double count = 0.0;

  if( x < 1 || x > NX1 || y_low >= y_high || ( y_low < 0.5 && y_high < 0.5 ) || ( y_low > ( NY1 + 0.5 ) && y_high > ( NY1 + 0.5 ) ) ){
    count = 0.0;
  }
  else{
    if( y_low < 0.5 )
      y_low = 0.5;
    if( y_high > ( NY1 + 0.5 ) )
      y_high = NY1 + 0.5;

    iy_low = y_low + 1.4999999999;
    iy_high = y_high - 0.4999999999;

    if( iy_low <= iy_high ){
      for( j=iy_low; j<=iy_high; j++ ){
        count += c[j-1][x-1];
      }
    }
    if( ( iy_high - iy_low ) > -2 ){
      if( (iy_low - 2) >= 0 )
	count += ( ( ( iy_low - 0.5 ) - y_low ) * c[iy_low-2][x-1] );
      if( iy_high <= ( NY1 - 1 ) )
	count += ( ( y_high - ( iy_high + 0.5 ) ) * c[iy_high][x-1] );
    }
    else{
      iy_high = iy_high + 1;
      iy_low = iy_low - 1;
      if( iy_high != iy_low ){
	printf( "Error in realcount\n" );
	return( 0.0 );
      }
      count += ( ( y_high - y_low ) * c[iy_high-1][x-1] );
    }
  }
  return( count );
}

void spmagnifyy( float count1[][NX1], double coef[][FITORDER], int nline,\
	        float count2[][NX1] ){

  int x, y;
  char buf[100];
  double ypeak[25], ypeakave, ypos[25], yposave;
     /* If the number of line of input file exceed 21, correct */
  double beta2, beta1, tmp1, tmp2;
  int i, j;
  double pos1, pos2;

  if( (int)FITORDER != 6 ){
    printf( "Error: Fitting order in 'hbs1to1x' was not 6.\n" );
    exit( -1 );
  }

  for( x=1; x<=NX1; x++ ){

    ypeakave = 0.0;
    yposave = 0.0;

    for( j=0; j<=(nline-1); j++ ){
      ypeak[j] = coef[j][0] + coef[j][1]*x + coef[j][2]*x*x + coef[j][3]*x*x*x\
	        + coef[j][4]*pow( x, 4.0 ) + coef[j][5]*pow( x, 5.0 );
      ypeakave += ypeak[j];
      if( NCCD == 1 )
        ypos[j] = 50.0 + j * 13.875 * 20 / (nline - 1); /* pix = width of a series = 3.0arcsec */
      else if( NCCD == 2 )
        ypos[j] = 25.0 + j * 6.94 * 20 / (nline - 1); /* pix = width of a series = 3.0arcsec */
      else
        //ypos[j] = 50.0 + j * 10.000 * 20 / (nline - 1); /* pix = width of a series = 3.0arcsec --trial5 */
      //ypos[j] = 50.0 + j * 13.875 * 20 / (nline - 1); /* pix = width of a series = 3.0arcsec --trial4 */
	// ypos[j] = 50.0 + j * 21.635 * 14 / (nline - 1); /* pix = width of a series = 3.0arcsec --trial3 */
        ypos[j] = 50.0 + j * 21.635 * 13 / (nline - 1); /* pix = width of a series = 3.0arcsec --trial3 */
      yposave += ypos[j];
    }
    ypeakave /= nline;
    yposave /= nline;

    tmp1 = 0.0;
    tmp2 = 0.0;

    for( j=0; j<=(nline-1); j++ ){
      tmp1 += ( ( ypeak[j] - ypeakave ) * ( ypos[j] - yposave ) );
      tmp2 += pow( ( ypos[j] - yposave ), 2.0 );
    }

    beta2 = tmp1 / tmp2;
    beta1 = ypeakave - beta2 * yposave;

    for( y=1; y<=NY1; y++ ){
      pos1 = beta1 + beta2 * ( y - 0.5 );
      pos2 = beta1 + beta2 * ( y + 0.5 );
      count2[y-1][x-1] = realcount1( count1, x, pos1, pos2 );
    }
  }
}

int writefile( char *outfile, float count[][NX1] ){

  fitsfile *fp;
  int i, status;
  char buf[100];
  long naxis1, naxis2, npix;
  char comment[100];

  status = 0;

  if( fits_open_file( &fp, outfile, READWRITE, &status ) != 0 ){
    printf( "Cannot open file '%s' with READWRITE mode. status=%d\n",\
	    outfile, status );
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS1", &naxis1,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS2", &naxis2,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( naxis1 != NX1 ){
    printf( "Error: definition of NX1 in 'hbsdefine.h'\n" );
    return( -1 );
  }
  if( naxis2 != NY1 ){
    printf( "Error: definition of y-dimension in 'hbsdefine.h'\n" );
    return( -1 );
  }
  npix = naxis1 * naxis2;

  if( fits_write_img( fp, TFLOAT, 1, npix, count, &status ) != 0 ){
    printf( "Error in the function fits_write_img\n" );
    return( status );
  }
  if( fits_close_file( fp, &status ) != 0 ){
    return( status );
  }
  return( 0 );
}

int main( int argc, char *argv[] ){

  double coef[25][FITORDER];
     /* If the number of line of input file exceed 21, correct */
  float count1[NY1][NX1];
  float count2[NY1][NX1];
  int i;
  char infile[50], outfile[50], fn_peakfit[50];
  int n_peak;

  if( argc != 4 ){
    printf( "usage: hbstfy infile fit_table outfile\n" );
    exit( -1 );
  }

  sprintf( infile, "%s", argv[1] );
  strcpy( fn_peakfit, argv[2] );
  strcpy( outfile, argv[3] );

  if( n_peak = readpeakfit( fn_peakfit, coef ), n_peak == 0 ){
    printf( "Read error in '%s'\n", fn_peakfit );
    printf( "Do 'hbs1to1x.cl' again !!\n" );
    return( 1 );
  }

  if( scanfile( infile, count1 ) != 0 ){
    printf( "read error in '%s'\n", infile );
    return( 1 );
  }

  spmagnifyy( count1, coef, n_peak, count2 );

  if( writefile( outfile, count2 ) != 0 ){
    printf( "write failed in '%s'\n", outfile );
    return( 1 );
  }

  return( 0 );
}
